// imports.gi
const GObject                = imports.gi.GObject
const Adw                    = imports.gi.Adw
const { SettingsBindFlags }  = imports.gi.Gio

// local modules
import { template_url }      from '../../utils/io.js'
import settings              from '../../utils/settings.js'
import constants             from '../../utils/constants.js'
import { connections }       from '../../utils/connections.js'
import { show_err_msg }      from '../../utils/prefs.js'
import appRow                from '../widgets/app-row.js'

// ----------------------------------------------------------------- end imports

// TODO: Setup guide

export const BlurEffect = GObject.registerClass (
    {
        Template: template_url (import.meta.url, './blureffect.ui'),
        GTypeName: 'BlurEffect',
        InternalChildren: [
            'opacity_ajustment',
            'sigma_ajustment',
            'add_row_btn',
            'blur_list',
        ],
    },
    class extends Adw.PreferencesPage {
        constructor () {
            super (...arguments)

            /** List to store into settings  */
            this.blur_list = []
        }

        _init () {
            super._init ()
            this.blur_list = settings ().blur_list

            for (const title of this.blur_list) {
                this._add_row (title)
            }

            connections ().connect (this._add_row_btn, 'clicked', () => {
                if (this.blur_list.includes ('')) {
                    return
                }
                this._add_row ()
                this.blur_list.push ('')
                settings ().blur_list = this.blur_list
            })

            settings ().bind (
                'blur-sigma',
                this._sigma_ajustment,
                'value',
                SettingsBindFlags.DEFAULT
            )
            settings ().bind (
                'blurred-window-opacity',
                this._opacity_ajustment,
                'value',
                SettingsBindFlags.DEFAULT
            )
        }

        _add_row (title = '') {
            const handler = {
                on_title_changed: (old_title, new_title) => {
                    if (this.blur_list.includes (new_title)) {
                        const title =
                            `[${new_title}] has been added into list` +
                            ', settings will not effect'
                        show_err_msg (this.root, title)
                        return false
                    }

                    const list = this.blur_list
                    list.splice (list.indexOf (old_title), 1, new_title)
                    settings ().blur_list = list

                    return true
                },
                on_delete: (row) => {
                    this._blur_list.remove (row)
                    const list = this.blur_list
                    list.splice (list.indexOf (row.title), 1)
                    settings ().blur_list = list
                },
            }

            const subtitle = title == '' ? constants.TIPS_EMPTY : ''
            this._blur_list.add (new appRow ({ title, subtitle }, handler))
        }
    }
)
