// local modules
import { _log } from './log.js'

// ---------------------------------------------------------------- [end import]

/**
 * This class is used to manager signal and handles of a object, one signal
 * only can be registered once.
 */
export class Connections {
    constructor () {
        var _a

        // -------------------------------------------------------- [public methods]

        /**
         * Map object to store signal sources and their handlers
         * @type {Map<GObject.Object, { [signal_name: string]: number }>}
         */
        this.connections = new Map ()
        try {
            throw new Error ()
        } catch (e) {
            _log (
                'Connection created in',
                (_a = e.stack) === null || _a === void 0
                    ? void 0
                    : _a
                        .split ('\n')
                        .map ((l) => l.split ('@')[0])
                        .join (',')
            )
        }
    }
    connect (source, ...[signal, cb]) {
        // Source has been added into manager
        {
            const handlers = this.connections.get (source)
            if (handlers !== undefined) {
                if (handlers[signal] !== undefined) {
                    const msg = `signal ${signal} of ${source} has been handled`
                    throw Error (msg)
                }
                handlers[signal] = source.connect (signal, cb)
                return
            }
        }

        // Source is first time register signal
        const handlers = {}
        handlers[signal] = source.connect (signal, cb)
        this.connections.set (source, handlers)
    }
    /** Disconnect signal for source */
    disconnect (source, signal) {
        const handlers = this.connections.get (source)
        if (handlers !== undefined) {
            const handler = handlers[signal]
            if (handler !== undefined) {
                source.disconnect (handler)
                delete handlers[signal]
                if (Object.keys (handler).length == 0) {
                    this.connections.delete (source)
                }
                return
            }
        }
    }

    disconnect_all (source) {
        // If provide source,  disconnect all signal of it
        if (source !== undefined) {
            const handlers = this.connections.get (source)
            if (handlers !== undefined) {
                Object.keys (handlers).forEach ((signal) => {
                    source.disconnect (handlers[signal])
                    delete handlers[signal]
                })
                this.connections.delete (source)
            }
            return
        }

        // otherwise clear signal for all objects.
        this.connections.forEach ((handlers, source) => {
            Object.keys (handlers).forEach ((signal) => {
                source.disconnect (handlers[signal])
                delete handlers[signal]
            })
        })
        this.connections.clear ()
    }
}

let _connections = null

/** A singleton of connections */
export const connections = () => {
    if (_connections === null) {
        _connections = new Connections ()
    }
    return _connections
}
